@@ -1,5 +1,5 @@
 # padding, used for unknown symbols and in training for no symbols
-DEFAULT_PADDING_SYMBOL = "_"
+# DEFAULT_PADDING_SYMBOL = "_"
 
 DEFAULT_CSV_SEPERATOR = "\t"
 
