@@ -217,16 +217,22 @@ class Encoder(nn.Module):
     - Bidirectional LSTM
   """
 
-  def __init__(self, hparams: HParams, stress_embedding_dim: Optional[int], tone_embedding_dim: Optional[int]):
+  def __init__(self, hparams: HParams, n_symbols: int, n_stresses: Optional[int], n_tones: Optional[int], n_durations: Optional[int]):
     super().__init__()
 
-    encoder_embedding_dim = hparams.symbols_embedding_dim
+    if hparams.train_symbol_with_embedding:
+      encoder_embedding_dim = hparams.symbols_embedding_dim
+    else:
+      encoder_embedding_dim = n_symbols
 
     if hparams.use_stress_embedding:
-      encoder_embedding_dim += stress_embedding_dim
+      encoder_embedding_dim += n_stresses
 
     if hparams.use_tone_embedding:
-      encoder_embedding_dim += tone_embedding_dim
+      encoder_embedding_dim += n_tones
+
+    if hparams.use_duration_embedding:
+      encoder_embedding_dim += n_durations
 
     convolutions = []
     for _ in range(hparams.encoder_n_convolutions):
@@ -284,7 +290,7 @@ class Encoder(nn.Module):
 
 
 class Decoder(nn.Module):
-  def __init__(self, hparams: HParams, stress_embedding_dim: Optional[int], tone_embedding_dim: Optional[int]):
+  def __init__(self, hparams: HParams, n_symbols: int, n_stresses: Optional[int], n_tones: Optional[int], n_durations: Optional[int], n_speakers: Optional[int]):
     super().__init__()
     self.n_mel_channels = hparams.n_mel_channels
     self.n_frames_per_step = hparams.n_frames_per_step
@@ -296,48 +302,59 @@ class Decoder(nn.Module):
 
     self.prenet = Prenet(hparams)
 
-    encoder_embedding_dim = hparams.symbols_embedding_dim
+    if hparams.train_symbol_with_embedding:
+      encoder_embedding_dim = hparams.symbols_embedding_dim
+    else:
+      encoder_embedding_dim = n_symbols
 
     if hparams.use_stress_embedding:
-      encoder_embedding_dim += stress_embedding_dim
+      encoder_embedding_dim += n_stresses
 
     if hparams.use_tone_embedding:
-      encoder_embedding_dim += tone_embedding_dim
+      encoder_embedding_dim += n_tones
+
+    if hparams.use_duration_embedding:
+      encoder_embedding_dim += n_durations
 
     lstm_hidden_size = ceil(encoder_embedding_dim / 2)
     lstm_out_dim = lstm_hidden_size * 2
 
-    self.merged_dimensions = lstm_out_dim
+    merged_dimensions = lstm_out_dim
 
     if hparams.use_speaker_embedding:
-      self.merged_dimensions += hparams.speakers_embedding_dim
+      if hparams.train_speaker_with_embedding:
+        merged_dimensions += hparams.speakers_embedding_dim
+      else:
+        merged_dimensions += n_speakers
 
     self.attention_rnn = nn.LSTMCell(
-        input_size=hparams.prenet_dim + self.merged_dimensions,
+        input_size=hparams.prenet_dim + merged_dimensions,
         hidden_size=hparams.attention_rnn_dim
     )
 
-    self.attention_layer = Attention(hparams, self.merged_dimensions)
+    self.attention_layer = Attention(hparams, merged_dimensions)
 
     # Deep Voice 2: "one site-speciﬁc embedding as the initial decoder GRU hidden state" -> is in Tacotron 2 now a LSTM
     self.decoder_rnn = nn.LSTMCell(
-        input_size=hparams.attention_rnn_dim + self.merged_dimensions,
+        input_size=hparams.attention_rnn_dim + merged_dimensions,
         hidden_size=hparams.decoder_rnn_dim,
         bias=True
     )
 
     self.linear_projection = LinearNorm(
-        in_dim=hparams.decoder_rnn_dim + self.merged_dimensions,
+        in_dim=hparams.decoder_rnn_dim + merged_dimensions,
         out_dim=hparams.n_mel_channels * hparams.n_frames_per_step
     )
 
     self.gate_layer = LinearNorm(
-        in_dim=hparams.decoder_rnn_dim + self.merged_dimensions,
+        in_dim=hparams.decoder_rnn_dim + merged_dimensions,
         out_dim=1,
         bias=True,
         w_init_gain='sigmoid'
     )
 
+    self.merged_dimensions = merged_dimensions
+
   def get_go_frame(self, memory):
     """ Gets all zeros frames to use as first decoder input
     PARAMS
@@ -562,75 +579,139 @@ class Decoder(nn.Module):
 
 
 ForwardXIn = Tuple[IntTensor, IntTensor, FloatTensor,
-                   IntTensor, Optional[IntTensor], Optional[LongTensor], Optional[LongTensor]]
+                   IntTensor, Optional[IntTensor], Optional[LongTensor], Optional[LongTensor], Optional[LongTensor]]
+
+
+def get_vector(tensor: IntTensor, use_embedding: bool, embedding: Optional[nn.Embedding], n_classes: Optional[int]) -> FloatTensor:
+  assert tensor is not None
+  if use_embedding:
+    assert embedding is not None
+    # shape: [70, 174, 512] -> [batch_size, maximum count of symbols, symbols_emb_dim]
+    result: FloatTensor = embedding(input=tensor)
+    assert result.dtype == torch.float32
+    # only in not eval grad
+    # assert result.requires_grad
+
+    if torch.isnan(result).any():
+      # embedding_inputs can be nan if training was not good
+      msg = "Embedding returned nan!"
+      logger = getLogger(__name__)
+      logger.error(msg)
+      raise Exception(msg)
+  else:
+    assert n_classes is not None
+    assert n_classes > 0
+    # Note: num_classes need to be defined because otherwise the dimension is not always the same since not all batches contain always all labels
+    result: LongTensor = F.one_hot(tensor, num_classes=n_classes)
+    result = result.type(torch.float32)
+    # both in eval and train no grad
+    assert not result.requires_grad
+  return result
 
 
 class Tacotron2(nn.Module):
-  def __init__(self, hparams: HParams, n_symbols: int, n_stresses: Optional[int], n_speakers: Optional[int], n_tones: Optional[int]):
+  def __init__(self, hparams: HParams, n_symbols: int, n_stresses: Optional[int], n_speakers: Optional[int], n_tones: Optional[int], n_durations: Optional[int]):
     super().__init__()
+    self.train_symbol_with_embedding = hparams.train_symbol_with_embedding
+    self.train_speaker_with_embedding = hparams.train_speaker_with_embedding
+    self.train_stress_with_embedding = hparams.train_stress_with_embedding
+    self.train_tone_with_embedding = hparams.train_tone_with_embedding
+    self.train_duration_with_embedding = hparams.train_duration_with_embedding
+
     self.use_speaker_embedding = hparams.use_speaker_embedding
     self.use_stress_embedding = hparams.use_stress_embedding
     self.use_tone_embedding = hparams.use_tone_embedding
+    self.use_duration_embedding = hparams.use_duration_embedding
 
     self.mask_padding = hparams.mask_padding
     self.n_mel_channels = hparams.n_mel_channels
 
-    # +1 because of padding
-    symbol_emb_weights = get_uniform_weights(
-        n_symbols, hparams.symbols_embedding_dim)
-    # rename will destroy all previous trained models
-    self.symbol_embeddings = weights_to_embedding(symbol_emb_weights)
+    self.symbol_embeddings = None
+    self.n_symbols = None
+    if hparams.train_symbol_with_embedding:
+      # +1 because of padding
+      symbol_emb_weights = get_uniform_weights(
+          n_symbols, hparams.symbols_embedding_dim)
+      # rename will destroy all previous trained models
+      self.symbol_embeddings = weights_to_embedding(symbol_emb_weights)
+    else:
+      self.n_symbols = n_symbols
 
-    if self.use_speaker_embedding:
+    if hparams.use_speaker_embedding:
       assert n_speakers is not None
-      speaker_emb_weights = get_xavier_weights(
-          n_speakers, hparams.speakers_embedding_dim)
-      self.speakers_embeddings = weights_to_embedding(
-          speaker_emb_weights)
+      self.speakers_embeddings = None
+      self.n_speakers = None
+      if hparams.train_speaker_with_embedding:
+        assert hparams.speakers_embedding_dim is not None
+        weights = get_xavier_weights(n_speakers, hparams.speakers_embedding_dim)
+        self.speakers_embeddings = weights_to_embedding(weights)
+      else:
+        self.n_speakers = n_speakers
 
-    self.stress_embedding_dim = None
-    if self.use_stress_embedding:
+    if hparams.use_stress_embedding:
       assert n_stresses is not None
-      self.stress_embedding_dim = n_stresses
+      self.stress_embedding = None
+      self.stress_embedding_dim = None
+      if hparams.train_stress_with_embedding:
+        assert hparams.stress_embedding_dim is not None
+        weights = get_xavier_weights(n_stresses, hparams.stress_embedding_dim)
+        self.stress_embedding = weights_to_embedding(weights)
+      else:
+        self.stress_embedding_dim = n_stresses
 
-    self.tone_embedding_dim = None
-    if self.use_tone_embedding:
+    if hparams.use_tone_embedding:
       assert n_tones is not None
-      self.tone_embedding_dim = n_tones
+      self.tone_embedding = None
+      self.tone_embedding_dim = None
+      if hparams.train_tone_with_embedding:
+        assert hparams.tone_embedding_dim is not None
+        weights = get_xavier_weights(n_tones, hparams.tone_embedding_dim)
+        self.tone_embedding = weights_to_embedding(weights)
+      else:
+        self.tone_embedding_dim = n_tones
 
-    self.encoder = Encoder(hparams, self.stress_embedding_dim, self.tone_embedding_dim)
-    self.decoder = Decoder(hparams, self.stress_embedding_dim, self.tone_embedding_dim)
+    if hparams.use_duration_embedding:
+      assert n_durations is not None
+      self.duration_embedding = None
+      self.duration_embedding_dim = None
+      if hparams.train_duration_with_embedding:
+        assert hparams.duration_embedding_dim is not None
+        weights = get_xavier_weights(n_durations, hparams.duration_embedding_dim)
+        self.duration_embedding = weights_to_embedding(weights)
+      else:
+        self.duration_embedding_dim = n_durations
+
+    self.encoder = Encoder(hparams, n_symbols, n_stresses,
+                           n_tones, n_durations)
+    self.decoder = Decoder(hparams, n_symbols, n_stresses,
+                           n_tones, n_durations, n_speakers)
     self.postnet = Postnet(hparams)
 
   def forward(self, inputs: ForwardXIn) -> Tuple[FloatTensor, FloatTensor, FloatTensor, FloatTensor]:
-    symbols, symbol_lengths, mels, output_lengths, speakers, stresses, tones = inputs
+    symbols, symbol_lengths, mels, output_lengths, speakers, stresses, tones, durations = inputs
     symbol_lengths, output_lengths = symbol_lengths.data, output_lengths.data
 
     # symbol_inputs: [70, 174] -> [batch_size, maximum count of symbols]
+    symbol_vector = get_vector(symbols, self.train_symbol_with_embedding,
+                               self.symbol_embeddings, self.n_symbols)
 
-    # shape: [70, 174, 512] -> [batch_size, maximum count of symbols, symbols_emb_dim]
-    embedded_inputs: FloatTensor = self.symbol_embeddings(input=symbols)
-    assert embedded_inputs.dtype == torch.float32
+    embedded_inputs = symbol_vector
 
     if self.use_stress_embedding:
-      assert stresses is not None
-      # Note: num_classes need to be defined because otherwise the dimension is not always the same since not all batches contain all stresses
-      stress_one_hot_tensor: LongTensor = F.one_hot(
-          stresses, num_classes=self.stress_embedding_dim)  # _, -, 0, 1, 2
-      stress_one_hot_tensor = stress_one_hot_tensor.type(torch.float32)
-      assert not stress_one_hot_tensor.requires_grad
-      embedded_inputs = torch.cat(
-          (embedded_inputs, stress_one_hot_tensor), -1)
+      stress_vector = get_vector(stresses, self.train_stress_with_embedding,
+                                 self.stress_embedding, self.stress_embedding_dim)
+      # concatenate embeddings (-1 means last dimension)
+      embedded_inputs = torch.cat((embedded_inputs, stress_vector), -1)
 
     if self.use_tone_embedding:
-      assert tones is not None
-      # Note: num_classes need to be defined because otherwise the dimension is not always the same since not all batches contain all tones
-      tones_one_hot_tensor: LongTensor = F.one_hot(
-          tones, num_classes=self.tone_embedding_dim)
-      tones_one_hot_tensor = tones_one_hot_tensor.type(torch.float32)
-      assert not tones_one_hot_tensor.requires_grad
-      embedded_inputs = torch.cat(
-          (embedded_inputs, tones_one_hot_tensor), -1)
+      tone_vector = get_vector(tones, self.train_tone_with_embedding,
+                               self.tone_embedding, self.tone_embedding_dim)
+      embedded_inputs = torch.cat((embedded_inputs, tone_vector), -1)
+
+    if self.use_duration_embedding:
+      duration_vector = get_vector(durations, self.train_duration_with_embedding,
+                                   self.duration_embedding, self.duration_embedding_dim)
+      embedded_inputs = torch.cat((embedded_inputs, duration_vector), -1)
 
     # swap last two dims
     embedded_inputs = embedded_inputs.transpose(1, 2)
@@ -643,12 +724,9 @@ class Tacotron2(nn.Module):
     merged_outputs = encoder_outputs
 
     if self.use_speaker_embedding:
-      assert speakers is not None
-      embedded_speakers: FloatTensor = self.speakers_embeddings(
-          input=speakers)
-      assert embedded_speakers.dtype == torch.float32
-      # concatenate symbol and speaker embeddings (-1 means last dimension)
-      merged_outputs = torch.cat((merged_outputs, embedded_speakers), -1)
+      speakers_vector = get_vector(speakers, self.train_speaker_with_embedding,
+                                   self.speakers_embeddings, self.n_speakers)
+      merged_outputs = torch.cat((merged_outputs, speakers_vector), -1)
 
     mel_outputs, gate_outputs, alignments = self.decoder(
         memory=merged_outputs,
@@ -670,34 +748,26 @@ class Tacotron2(nn.Module):
 
     return mel_outputs, mel_outputs_postnet, gate_outputs, alignments
 
-  def inference(self, symbols: IntTensor, stresses: Optional[LongTensor], tones: Optional[LongTensor], speakers: Optional[IntTensor], max_decoder_steps: int) -> Tuple[Tensor, Tensor, Tensor, Tensor]:
-    embedded_inputs: FloatTensor = self.symbol_embeddings(input=symbols)
-    assert embedded_inputs.dtype == torch.float32
+  def inference(self, symbols: IntTensor, stresses: Optional[LongTensor], tones: Optional[LongTensor], durations: Optional[LongTensor], speakers: Optional[IntTensor], max_decoder_steps: int) -> Tuple[Tensor, Tensor, Tensor, Tensor]:
+    symbol_vector = get_vector(symbols, self.train_symbol_with_embedding,
+                               self.symbol_embeddings, self.n_symbols)
 
-    if torch.isnan(embedded_inputs).any():
-      # embedding_inputs can be nan if training was not good
-      msg = "Symbol embeddings returned nan!"
-      logger = getLogger(__name__)
-      logger.error(msg)
-      raise Exception(msg)
+    embedded_inputs = symbol_vector
 
     if self.use_stress_embedding:
-      assert stresses is not None
-      # Note: num_classes need to be defined because otherwise the dimension is not always the same since not all batches contain all stresses
-      stress_embeddings: LongTensor = F.one_hot(
-          stresses, num_classes=self.stress_embedding_dim)  # _, -, 0, 1, 2
-      stress_embeddings = stress_embeddings.type(torch.float32)
-      embedded_inputs = torch.cat(
-          (embedded_inputs, stress_embeddings), -1)
+      stress_vector = get_vector(stresses, self.train_stress_with_embedding,
+                                 self.stress_embedding, self.stress_embedding_dim)
+      embedded_inputs = torch.cat((embedded_inputs, stress_vector), -1)
 
     if self.use_tone_embedding:
-      assert tones is not None
-      # Note: num_classes need to be defined because otherwise the dimension is not always the same since not all batches contain all tones
-      tone_embeddings: LongTensor = F.one_hot(
-          tones, num_classes=self.tone_embedding_dim)
-      tone_embeddings = tone_embeddings.type(torch.float32)
-      embedded_inputs = torch.cat(
-          (embedded_inputs, tone_embeddings), -1)
+      tone_vector = get_vector(tones, self.train_tone_with_embedding,
+                               self.tone_embedding, self.tone_embedding_dim)
+      embedded_inputs = torch.cat((embedded_inputs, tone_vector), -1)
+
+    if self.use_duration_embedding:
+      duration_vector = get_vector(durations, self.train_duration_with_embedding,
+                                   self.duration_embedding, self.duration_embedding_dim)
+      embedded_inputs = torch.cat((embedded_inputs, duration_vector), -1)
 
     # swap last two dims
     embedded_inputs = embedded_inputs.transpose(1, 2)
@@ -707,12 +777,9 @@ class Tacotron2(nn.Module):
     merged_outputs = encoder_outputs
 
     if self.use_speaker_embedding:
-      assert speakers is not None
-      embedded_speakers: FloatTensor = self.speakers_embeddings(
-          input=speakers)
-      assert embedded_speakers.dtype == torch.float32
-      # concatenate symbol and speaker embeddings (-1 means last dimension)
-      merged_outputs = torch.cat((merged_outputs, embedded_speakers), -1)
+      speakers_vector = get_vector(speakers, self.train_speaker_with_embedding,
+                                   self.speakers_embeddings, self.n_speakers)
+      merged_outputs = torch.cat((merged_outputs, speakers_vector), -1)
 
     decoder_outputs, reached_max_decoder_steps = self.decoder.inference(
         merged_outputs, max_decoder_steps)
