@@ -9,7 +9,7 @@ from logging import Logger, getLogger
 from math import floor, sqrt
 from multiprocessing import cpu_count
 from pathlib import Path
-from typing import Dict, Generator, Iterable, List, Optional, Set, Tuple, Type, TypeVar, Union
+from typing import Any, Dict, Generator, Iterable, List, Optional, Set, Tuple, Type, TypeVar, Union
 
 import matplotlib.ticker as ticker
 import numpy as np
@@ -17,7 +17,7 @@ import torch
 from matplotlib import pyplot as plt
 from matplotlib.figure import Figure
 from scipy.spatial.distance import cosine
-from torch import IntTensor, Tensor, nn
+from torch import IntTensor, Tensor, nn  # pylint: disable=no-name-in-module
 from torch.nn import Module
 
 from tacotron.globals import SPACE_DISPLAYABLE
@@ -39,6 +39,29 @@ formatter = logging.Formatter(
 )
 
 
+def get_items_by_index(it: Iterable[Any], indices: Set[Any]) -> Generator[Any, None, None]:
+  for i, item in enumerate(it):
+    if i in indices:
+      yield item
+
+
+def find_indices(it: Iterable[Any], find: Set[Any]) -> Generator[int, None, None]:
+  for i, item in enumerate(it):
+    if item in find:
+      yield i
+
+
+def cut_string(s: str, cut: Set[str]) -> Tuple[str, str]:
+  cut_str = ""
+  rest_str = ""
+  for character in s:
+    if character in cut:
+      cut_str += character
+    else:
+      rest_str += character
+  return rest_str, cut_str
+
+
 def get_default_logger():
   return logging.getLogger("default")
 
