@@ -1,25 +1,27 @@
-from tacotron.tone_detection import split_phoneme_and_tone
+
+
+from tacotron.frontend.main import split_tone
 
 
 def test_vowel_toned__returns_vowel_tone():
-  phoneme, tone = split_phoneme_and_tone("i˧˩˧")
+  phoneme, tone = split_tone("i˧˩˧")
   assert phoneme == "i"
   assert tone == "˧˩˧"
 
 
 def test_diphthong_toned__returns_vowel_tone():
-  phoneme, tone = split_phoneme_and_tone("ai˩˧")
+  phoneme, tone = split_tone("ai˩˧")
   assert phoneme == "ai"
   assert tone == "˩˧"
 
 
 def test_consonant__returns_consonant_empty():
-  phoneme, tone = split_phoneme_and_tone("b")
+  phoneme, tone = split_tone("b")
   assert phoneme == "b"
-  assert tone == ""
+  assert tone == "-"
 
 
 def test_consonant_toned__returns_consonant_empty():
-  phoneme, tone = split_phoneme_and_tone("b˧")
+  phoneme, tone = split_tone("b˧")
   assert phoneme == "b"
   assert tone == "˧"
